@@ -13,16 +13,9 @@ import tempfile
 import zipfile
 
 from bypy.constants import PREFIX, PYTHON, SW, python_major_minor_version
-from bypy.freeze import (
-    extract_extension_modules, freeze_python, path_to_freeze_dir
-)
-from bypy.macos_sign import (
-    codesign, create_entitlements_file, make_certificate_useable, notarize_app,
-    verify_signature
-)
-from bypy.utils import (
-    current_dir, mkdtemp, py_compile, run_shell, timeit, walk
-)
+from bypy.freeze import extract_extension_modules, freeze_python, path_to_freeze_dir
+from bypy.macos_sign import codesign, create_entitlements_file, make_certificate_useable, notarize_app, verify_signature
+from bypy.utils import current_dir, mkdtemp, py_compile, run_shell, timeit, walk
 
 iv = globals()['init_env']
 kitty_constants = iv['kitty_constants']
@@ -113,6 +106,9 @@ def do_sign(app_dir):
             codesign(fw)
             items = set(os.listdir('.')) - fw
             codesign(expand_dirs(items))
+        # Sign kitten
+        with current_dir('MacOS'):
+            codesign('kitten')
 
     # Now sign the main app
     codesign(app_dir)
@@ -171,6 +167,7 @@ class Freeze(object):
         self.add_misc_libraries()
         self.freeze_python()
         self.add_ca_certs()
+        self.build_frozen_tools()
         if not self.dont_strip:
             self.strip_files()
         if not self.skip_tests:
@@ -378,6 +375,10 @@ class Freeze(object):
             if f.endswith('.so') or f.endswith('.dylib'):
                 self.fix_dependencies_in_lib(f)
 
+    @flush
+    def build_frozen_tools(self):
+        iv['build_frozen_tools'](join(self.contents_dir, 'MacOS', 'kitty'))
+
     @flush
     def add_site_packages(self):
         print('\nAdding site-packages')
