@@ -699,25 +699,6 @@ cocoa_update_menu_bar_title(PyObject *pytitle) {
     [m release];
 } // }}}
 
-bool
-cocoa_make_window_resizable(void *w, bool resizable) {
-    NSWindow *window = (NSWindow*)w;
-
-    @try {
-        if (resizable) {
-            [window setStyleMask:
-                [window styleMask] | NSWindowStyleMaskResizable];
-        } else {
-            [window setStyleMask:
-                [window styleMask] & ~NSWindowStyleMaskResizable];
-        }
-    } @catch (NSException *e) {
-        log_error("Failed to set style mask: %s: %s", [[e name] UTF8String], [[e reason] UTF8String]);
-        return false;
-    }
-    return true;
-}
-
 #define NSLeftAlternateKeyMask  (0x000020 | NSEventModifierFlagOption)
 #define NSRightAlternateKeyMask (0x000040 | NSEventModifierFlagOption)
 
@@ -756,6 +737,21 @@ cocoa_focus_window(void *w) {
     [window makeKeyWindow];
 }
 
+bool
+cocoa_set_shadow(void *w, bool has_shadow) {
+    NSWindow *window = (NSWindow*)w;
+    bool current = window.hasShadow;
+    [window setHasShadow:has_shadow];
+    if (has_shadow) {  // invalidate the shadow to clear the ghosting
+        [window invalidateShadow];
+        // invalidateShadow does not work with layer backed views, see http://www.openradar.me/34184270
+        // so do the frame resize dance to force AppKit to invalidate the shadow even with the layer
+        [window setFrame:NSMakeRect(NSMinX(window.frame), NSMinY(window.frame), NSWidth(window.frame)+1, NSHeight(window.frame)) display:NO animate:NO];
+        [window setFrame:NSMakeRect(NSMinX(window.frame), NSMinY(window.frame), NSWidth(window.frame)-1, NSHeight(window.frame)) display:NO animate:NO];
+    }
+    return current;
+}
+
 long
 cocoa_window_number(void *w) {
     NSWindow *window = (NSWindow*)w;
@@ -782,19 +778,10 @@ cocoa_get_workspace_ids(void *w, size_t *workspace_ids, size_t array_sz) {
 static PyObject*
 cocoa_get_lang(PyObject UNUSED *self) {
     @autoreleasepool {
-
-    NSString* locale = nil;
-    NSString* lang_code = [[NSLocale currentLocale] objectForKey:NSLocaleLanguageCode];
+    NSString* lang_code = [[NSLocale currentLocale] languageCode];
     NSString* country_code = [[NSLocale currentLocale] objectForKey:NSLocaleCountryCode];
-    if (lang_code && country_code) {
-        locale = [NSString stringWithFormat:@"%@_%@", lang_code, country_code];
-    } else {
-        locale = [[NSLocale currentLocale] localeIdentifier];
-    }
-    if (!locale) { Py_RETURN_NONE; }
-    const char* locale_utf8 = [locale UTF8String];
-    return Py_BuildValue("s", locale_utf8);
-
+    NSString* identifier = [[NSLocale currentLocale] localeIdentifier];
+    return Py_BuildValue("sss", lang_code ? [lang_code UTF8String]:"", country_code ? [country_code UTF8String] : "", identifier ? [identifier UTF8String]: "");
     } // autoreleasepool
 }
 
@@ -822,46 +809,6 @@ cocoa_set_activation_policy(bool hide_from_tasks) {
     [NSApp setActivationPolicy:(hide_from_tasks ? NSApplicationActivationPolicyAccessory : NSApplicationActivationPolicyRegular)];
 }
 
-void
-cocoa_set_titlebar_appearance(void *w, unsigned int theme)
-{
-    if (!theme) return;
-    @autoreleasepool {
-        NSWindow *window = (NSWindow*)w;
-        [window setAppearance:[NSAppearance appearanceNamed:((theme == 2) ? NSAppearanceNameVibrantDark : NSAppearanceNameVibrantLight)]];
-    } // autoreleasepool
-}
-
-void
-cocoa_set_titlebar_color(void *w, color_type titlebar_color)
-{
-    @autoreleasepool {
-
-    NSWindow *window = (NSWindow*)w;
-
-    double red = ((titlebar_color >> 16) & 0xFF) / 255.0;
-    double green = ((titlebar_color >> 8) & 0xFF) / 255.0;
-    double blue = (titlebar_color & 0xFF) / 255.0;
-
-    NSColor *background =
-        [NSColor colorWithSRGBRed:red
-                            green:green
-                             blue:blue
-                            alpha:1.0];
-    [window setTitlebarAppearsTransparent:YES];
-    [window setBackgroundColor:background];
-
-    double luma = 0.2126 * red + 0.7152 * green + 0.0722 * blue;
-
-    if (luma < 0.5) {
-        [window setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantDark]];
-    } else {
-        [window setAppearance:[NSAppearance appearanceNamed:NSAppearanceNameVibrantLight]];
-    }
-
-    } // autoreleasepool
-}
-
 static PyObject*
 cocoa_set_url_handler(PyObject UNUSED *self, PyObject *args) {
     @autoreleasepool {
@@ -978,17 +925,6 @@ cleanup(void) {
     } // autoreleasepool
 }
 
-void
-cocoa_hide_window_title(void *w)
-{
-    @autoreleasepool {
-
-    NSWindow *window = (NSWindow*)w;
-    [window setTitleVisibility:NSWindowTitleHidden];
-
-    } // autoreleasepool
-}
-
 void
 cocoa_system_beep(const char *path) {
     if (!path) { NSBeep(); return; }
