@@ -289,17 +289,64 @@ window_iconify_callback(GLFWwindow *window, int iconified) {
     global_state.callback_os_window = NULL;
 }
 
+#ifdef __APPLE__
+static void
+cocoa_out_of_sequence_render(OSWindow *window) {
+    make_os_window_context_current(window);
+    window->needs_render = true;
+    bool rendered = render_os_window(window, monotonic(), true, true);
+    if (!rendered) {
+        blank_os_window(window);
+        swap_window_buffers(window);
+    }
+    window->needs_render = true;
+}
+
+static void
+cocoa_os_window_resized(GLFWwindow *w) {
+    if (!set_callback_window(w)) return;
+    if (global_state.callback_os_window->ignore_resize_events) return;
+    cocoa_out_of_sequence_render(global_state.callback_os_window);
+    global_state.callback_os_window = NULL;
+}
+#endif
+
+
+
 void
 change_live_resize_state(OSWindow *w, bool in_progress) {
     if (in_progress != w->live_resize.in_progress) {
         w->live_resize.in_progress = in_progress;
+        w->live_resize.num_of_resize_events = 0;
         apply_swap_interval(in_progress ? 0 : -1);
+#ifdef __APPLE__
+        extern bool cocoa_set_shadow(void*, bool);
+        if (w->is_semi_transparent && w->handle) {
+            // shadow with transparency during resize causes burn-in of window contents
+            // aka ghosting of the text after resize is completed. So avoid that by turning
+            // the shadow on and off.
+            if (in_progress) {
+                bool had_shadow = cocoa_set_shadow(glfwGetCocoaWindow(w->handle), false);
+                w->has_cocoa_shadow = had_shadow;
+            } else {
+                if (w->has_cocoa_shadow) {
+                    // turning on the shadow will cause a resize event which will be delivered before cocoa_set_shadow returns
+                    w->ignore_resize_events = true;
+                    cocoa_set_shadow(glfwGetCocoaWindow(w->handle), true);
+                    w->ignore_resize_events = false;
+                    w->has_cocoa_shadow = false;
+                }
+            }
+        }
+        cocoa_out_of_sequence_render(w);
+#endif
     }
 }
 
 static void
 live_resize_callback(GLFWwindow *w, bool started) {
     if (!set_callback_window(w)) return;
+    if (global_state.callback_os_window->ignore_resize_events) return;
     global_state.callback_os_window->live_resize.from_os_notification = true;
     change_live_resize_state(global_state.callback_os_window, true);
     global_state.has_pending_resizes = true;
@@ -313,6 +360,7 @@ live_resize_callback(GLFWwindow *w, bool started) {
 static void
 framebuffer_size_callback(GLFWwindow *w, int width, int height) {
     if (!set_callback_window(w)) return;
+    if (global_state.callback_os_window->ignore_resize_events) return;
     int min_width, min_height; min_size_for_os_window(global_state.callback_os_window, &min_width, &min_height);
     if (width >= min_width && height >= min_height) {
         OSWindow *window = global_state.callback_os_window;
@@ -331,6 +379,7 @@ framebuffer_size_callback(GLFWwindow *w, int width, int height) {
 static void
 dpi_change_callback(GLFWwindow *w, float x_scale UNUSED, float y_scale UNUSED) {
     if (!set_callback_window(w)) return;
+    if (global_state.callback_os_window->ignore_resize_events) return;
     // Ensure update_os_window_viewport() is called in the near future, it will
     // take care of DPI changes.
     OSWindow *window = global_state.callback_os_window;
@@ -871,9 +920,9 @@ init_window_chrome_state(WindowChromeState *s, color_type active_window_bg, bool
     s->background_opacity = background_opacity;
 }
 
-#ifdef __APPLE__
 static void
 apply_window_chrome_state(GLFWwindow *w, WindowChromeState new_state, int width, int height, bool window_decorations_changed) {
+#ifdef __APPLE__
     glfwCocoaSetWindowChrome(w,
         new_state.color, new_state.use_system_color, new_state.system_color,
         new_state.background_blur, new_state.hide_window_decorations,
@@ -882,8 +931,19 @@ apply_window_chrome_state(GLFWwindow *w, WindowChromeState new_state, int width,
     );
     // Need to resize the window again after hiding decorations or title bar to take up screen space
     if (window_decorations_changed) glfwSetWindowSize(w, width, height);
-}
+#else
+        if (window_decorations_changed) {
+            bool hide_window_decorations = new_state.hide_window_decorations & 1;
+            glfwSetWindowAttrib(w, GLFW_DECORATED, !hide_window_decorations);
+            glfwSetWindowSize(w, width, height);
+        }
+        if (global_state.is_wayland) {
+            if (glfwWaylandSetTitlebarColor) glfwWaylandSetTitlebarColor(w, new_state.color, new_state.use_system_color);
+        } else {
+            glfwSetX11WindowBlurred(w, new_state.background_blur > 0);
+        }
 #endif
+}
 
 void
 set_os_window_chrome(OSWindow *w) {
@@ -906,20 +966,7 @@ set_os_window_chrome(OSWindow *w) {
         int width, height;
         glfwGetWindowSize(w->handle, &width, &height);
         bool window_decorations_changed = new_state.hide_window_decorations != w->last_window_chrome.hide_window_decorations;
-#ifdef __APPLE__
         apply_window_chrome_state(w->handle, new_state, width, height, window_decorations_changed);
-#else
-        if (window_decorations_changed) {
-            bool hide_window_decorations = new_state.hide_window_decorations & 1;
-            glfwSetWindowAttrib(w->handle, GLFW_DECORATED, !hide_window_decorations);
-            glfwSetWindowSize(w->handle, width, height);
-        }
-        if (global_state.is_wayland) {
-            if (glfwWaylandSetTitlebarColor) glfwWaylandSetTitlebarColor(w->handle, new_state.color, new_state.use_system_color);
-        } else {
-            glfwSetX11WindowBlurred(w->handle, new_state.background_blur > 0);
-        }
-#endif
         w->last_window_chrome = new_state;
     }
 }
@@ -1083,6 +1130,7 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
 #ifdef __APPLE__
     if (OPT(macos_option_as_alt)) glfwSetCocoaTextInputFilter(glfw_window, filter_option);
     glfwSetCocoaToggleFullscreenIntercept(glfw_window, intercept_cocoa_fullscreen);
+    glfwCocoaSetWindowResizeCallback(glfw_window, cocoa_os_window_resized);
 #endif
     send_prerendered_sprites_for_window(w);
     if (logo.pixels && logo.width && logo.height) glfwSetWindowIcon(glfw_window, 1, &logo);
@@ -1121,6 +1169,8 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
     init_window_chrome_state(&w->last_window_chrome, OPT(background), w->is_semi_transparent, w->background_opacity);
 #ifdef __APPLE__
     apply_window_chrome_state(w->handle, w->last_window_chrome, width, height, OPT(hide_window_decorations) != 0);
+#else
+    apply_window_chrome_state(w->handle, w->last_window_chrome, width, height, false);
 #endif
     // Update window state
     // We do not call glfwWindowHint to set GLFW_MAXIMIZED before the window is created.
