@@ -65,7 +65,7 @@ bglayout(PyObject *layout_name) {
         case 'm': return MIRRORED;
         case 's': return SCALED;
         case 'c': {
-            return name[1] == 'l' ? CLAMPED : CENTER_CLAMPED;
+            return name[1] == 'l' ? CLAMPED : (name[1] == 's' ? CENTER_SCALED : CENTER_CLAMPED);
         }
         default: break;
     }
