@@ -427,8 +427,22 @@ draw_background_image(OSWindow *w) {
 #else
     int window_width = w->viewport_width, window_height = w->viewport_height;
 #endif
+    GLfloat iwidth = (GLfloat)w->bgimage->width;
+    GLfloat iheight = (GLfloat)w->bgimage->height;
+    GLfloat vwidth = (GLfloat)window_width;
+    GLfloat vheight = (GLfloat)window_height;
+    if (CENTER_SCALED == OPT(background_image_layout)) {
+        GLfloat ifrac = iwidth / iheight;
+        if (ifrac > (vwidth / vheight)) {
+            iheight = vheight;
+            iwidth = iheight * ifrac;
+        } else {
+            iwidth = vwidth;
+            iheight = iwidth / ifrac;
+        }
+    }
     glUniform4f(bgimage_program_layout.uniforms.sizes,
-        (GLfloat)window_width, (GLfloat)window_height, (GLfloat)w->bgimage->width, (GLfloat)w->bgimage->height);
+        vwidth, vheight, iwidth, iheight);
     glUniform1f(bgimage_program_layout.uniforms.premult, w->is_semi_transparent ? 1.f : 0.f);
     GLfloat tiled = 0.f;;
     GLfloat left = -1.0, top = 1.0, right = 1.0, bottom = -1.0;
@@ -436,18 +450,16 @@ draw_background_image(OSWindow *w) {
         case TILING: case MIRRORED: case CLAMPED:
             tiled = 1.f; break;
         case SCALED:
-            tiled = 0.f; break;
-        case CENTER_CLAMPED:
-            tiled = 1.f;
-            if (window_width > (int)w->bgimage->width) {
-                GLfloat frac = (window_width - w->bgimage->width) / (GLfloat)window_width;
-                left += frac; right += frac;
-            }
-            if (window_height > (int)w->bgimage->height) {
-                GLfloat frac = (window_height - w->bgimage->height) / (GLfloat)window_height;
-                top -= frac; bottom -= frac;
-            }
             break;
+        case CENTER_CLAMPED:
+        case CENTER_SCALED: {
+            GLfloat wfrac = (vwidth - iwidth) / vwidth;
+            GLfloat hfrac = (vheight - iheight) / vheight;
+            left += wfrac;
+            right -= wfrac;
+            top -= hfrac;
+            bottom += hfrac;
+        } break;
     }
     glUniform1f(bgimage_program_layout.uniforms.tiled, tiled);
     glUniform4f(bgimage_program_layout.uniforms.positions, left, top, right, bottom);
@@ -457,49 +469,24 @@ draw_background_image(OSWindow *w) {
     unbind_program();
 }
 
-static void
-fit_image_in_viewport(GraphicsUniforms *u, ImageRenderData *rd, ImageRect viewport) {
-    // dest_rect: x: -1 to 1, y: 1 to -1. src_rect: x: 0 to 1 for both axes
-    ImageRect src_rect = rd->src_rect, dest_rect = rd->dest_rect;
-    const float img_width = dest_rect.right - dest_rect.left;
-    if (viewport.left > dest_rect.left) {
-        float frac = 1.f - (dest_rect.right - viewport.left) / img_width;
-        dest_rect.left = viewport.left;
-        src_rect.left += frac; // src_rect: left is 0 right is 1
-    }
-    if (viewport.right < dest_rect.right) {
-        float frac = 1.f - (viewport.right - dest_rect.left) / img_width;
-        dest_rect.right = viewport.right;
-        src_rect.right -= frac; // src_rect: left is 0 right is 1
-    }
-    const float img_height = dest_rect.top - dest_rect.bottom;
-    if (viewport.top < dest_rect.top) {
-        float frac = 1.f - (viewport.top - dest_rect.bottom) / img_height;
-        dest_rect.top = viewport.top;
-        src_rect.top += frac; // src_rect.bottom is 1 top is 0
-    }
-    if (viewport.bottom > dest_rect.bottom) {
-        float frac = 1.f - (dest_rect.top - viewport.bottom) / img_height;
-        dest_rect.bottom = viewport.bottom;
-        src_rect.bottom -= frac; // src_rect.bottom is 1 top is 0
-    }
-    glUniform4f(u->src_rect, src_rect.left, src_rect.top, src_rect.right, src_rect.bottom);
-    glUniform4f(u->dest_rect, dest_rect.left, dest_rect.top, dest_rect.right, dest_rect.bottom);
-}
-
 static void
 draw_graphics(int program, ssize_t vao_idx, ImageRenderData *data, GLuint start, GLuint count, ImageRect viewport) {
     bind_program(program);
     glActiveTexture(GL_TEXTURE0 + GRAPHICS_UNIT);
     GraphicsUniforms *u = &graphics_program_layouts[program].uniforms;
+    glUniform4f(u->viewport, viewport.left, viewport.top, viewport.right, viewport.bottom);
+    glEnable(GL_CLIP_DISTANCE0); glEnable(GL_CLIP_DISTANCE1); glEnable(GL_CLIP_DISTANCE2); glEnable(GL_CLIP_DISTANCE3);
     for (GLuint i=0; i < count;) {
-        ImageRenderData *rd = data + start + i;
-        glBindTexture(GL_TEXTURE_2D, rd->texture_id);
-        for (GLuint k=0; k < rd->group_count; k++, i++) {
-            fit_image_in_viewport(u, rd, viewport);
+        ImageRenderData *group = data + start + i;
+        glBindTexture(GL_TEXTURE_2D, group->texture_id);
+        for (GLuint k=0; k < group->group_count; k++, i++) {
+            ImageRenderData *rd = data + start + i;
+            glUniform4f(u->src_rect, rd->src_rect.left, rd->src_rect.top, rd->src_rect.right, rd->src_rect.bottom);
+            glUniform4f(u->dest_rect, rd->dest_rect.left, rd->dest_rect.top, rd->dest_rect.right, rd->dest_rect.bottom);
             glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
         }
     }
+    glDisable(GL_CLIP_DISTANCE0); glDisable(GL_CLIP_DISTANCE1); glDisable(GL_CLIP_DISTANCE2); glDisable(GL_CLIP_DISTANCE3);
     bind_vertex_array(vao_idx);
 }
 
