@@ -11,6 +11,7 @@ import (
 	"kitty/kittens/hints"
 	"kitty/kittens/hyperlinked_grep"
 	"kitty/kittens/icat"
+	"kitty/kittens/show_key"
 	"kitty/kittens/ssh"
 	"kitty/kittens/themes"
 	"kitty/kittens/unicode_input"
@@ -18,6 +19,8 @@ import (
 	"kitty/tools/cmd/at"
 	"kitty/tools/cmd/edit_in_kitty"
 	"kitty/tools/cmd/pytest"
+	"kitty/tools/cmd/run_shell"
+	"kitty/tools/cmd/show_error"
 	"kitty/tools/cmd/update_self"
 	"kitty/tools/tui"
 )
@@ -27,6 +30,7 @@ var _ = fmt.Print
 func KittyToolEntryPoints(root *cli.Command) {
 	root.Add(cli.OptionSpec{
 		Name: "--version", Type: "bool-set", Help: "The current kitten version."})
+	tui.PrepareRootCmd(root)
 	// @
 	at.EntryPoint(root)
 	// update-self
@@ -41,6 +45,8 @@ func KittyToolEntryPoints(root *cli.Command) {
 	ssh.EntryPoint(root)
 	// unicode_input
 	unicode_input.EntryPoint(root)
+	// show_key
+	show_key.EntryPoint(root)
 	// hyperlinked_grep
 	hyperlinked_grep.EntryPoint(root)
 	// ask
@@ -51,6 +57,11 @@ func KittyToolEntryPoints(root *cli.Command) {
 	diff.EntryPoint(root)
 	// themes
 	themes.EntryPoint(root)
+	themes.ParseEntryPoint(root)
+	// run-shell
+	run_shell.EntryPoint(root)
+	// show_error
+	show_error.EntryPoint(root)
 	// __pytest__
 	pytest.EntryPoint(root)
 	// __hold_till_enter__
@@ -63,4 +74,13 @@ func KittyToolEntryPoints(root *cli.Command) {
 			return
 		},
 	})
+	// __confirm_and_run_shebang__
+	root.AddSubCommand(&cli.Command{
+		Name:            "__confirm_and_run_shebang__",
+		Hidden:          true,
+		OnlyArgsAllowed: true,
+		Run: func(cmd *cli.Command, args []string) (rc int, err error) {
+			return confirm_and_run_shebang(args)
+		},
+	})
 }
