@@ -4,8 +4,9 @@ package loop
 
 import (
 	"fmt"
-	"sort"
 	"time"
+
+	"golang.org/x/exp/slices"
 )
 
 func (self *Loop) add_timer(interval time.Duration, repeats bool, callback TimerCallback) (IdType, error) {
@@ -34,10 +35,8 @@ func (self *Loop) remove_timer(id IdType) bool {
 }
 
 func (self *Loop) dispatch_timers(now time.Time) error {
-	updated := false
 	self.timers_temp = self.timers_temp[:0]
-	self.timers_temp = append(self.timers_temp, self.timers...)
-	for i, t := range self.timers_temp {
+	for _, t := range self.timers {
 		if now.After(t.deadline) {
 			err := t.callback(t.id)
 			if err != nil {
@@ -45,18 +44,18 @@ func (self *Loop) dispatch_timers(now time.Time) error {
 			}
 			if t.repeats {
 				t.update_deadline(now)
-				updated = true
-			} else {
-				self.timers = append(self.timers[:i], self.timers[i+1:]...)
+				self.timers_temp = append(self.timers_temp, t)
 			}
 		}
 	}
-	if updated {
+	self.timers = self.timers[:len(self.timers_temp)]
+	if len(self.timers) > 0 {
+		copy(self.timers, self.timers_temp)
 		self.sort_timers()
 	}
 	return nil
 }
 
 func (self *Loop) sort_timers() {
-	sort.SliceStable(self.timers, func(a, b int) bool { return self.timers[a].deadline.Before(self.timers[b].deadline) })
+	slices.SortStableFunc(self.timers, func(a, b *timer) bool { return a.deadline.Before(b.deadline) })
 }
