@@ -10,7 +10,7 @@ import (
 
 var _ = fmt.Print
 
-type Once[T any] struct {
+type once[T any] struct {
 	done       uint32
 	mutex      sync.Mutex
 	cached_val T
@@ -18,14 +18,14 @@ type Once[T any] struct {
 	Run func() T
 }
 
-func (self *Once[T]) Get() T {
+func (self *once[T]) Get() T {
 	if atomic.LoadUint32(&self.done) == 0 {
 		self.do_slow()
 	}
 	return self.cached_val
 }
 
-func (self *Once[T]) do_slow() {
+func (self *once[T]) do_slow() {
 	self.mutex.Lock()
 	defer self.mutex.Unlock()
 	if atomic.LoadUint32(&self.done) == 0 {
@@ -33,3 +33,7 @@ func (self *Once[T]) do_slow() {
 		self.cached_val = self.Run()
 	}
 }
+
+func Once[T any](f func() T) func() T {
+	return (&once[T]{Run: f}).Get
+}
